/***********************************************************************
 *
 * Copyright (C) 2004 by FS Forth-Systeme GmbH.
 * All rights reserved.
 *
 * $Id: ns9750_bbus.h,v 1.2 2005/09/06 07:35:00 uzeisberger Exp $
 * @Author: Markus Pietrek, Thomas Kuhlmann
 * @Descr: Definitions for BBus usage
 * @References: [1] NS9750 Hardware Reference Manual/December 2003 Chap. 10
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 *
 ***********************************************************************/
/***********************************************************************
 *  @History:
 *     2004/09/07 : missing define for CTRL and STAT added
 *     2004/09/07 : volatile access replaced by rmwl()
 *     2004/08/17 : removed addr_bbus
 ***********************************************************************/

#ifndef FS_NS9750_BBUS_H
#define FS_NS9750_BBUS_H

#define NS_BBUS_MODULE_BASE		(0xF9000000) /* virtual address */
#define NS_BBUS_UTILITY_BASE		(0xF9600000) /* virtual address */

/* We have support for 72 GPIO pins */

#define addr_gpio_cfg(pin) ( NS_BBUS_GPIO_CFG_BASE + (((pin) >> 3) * 4) \
							+ (((pin) <56)?0:212))

/* To Read/Modify/Write a pin configuration register, use it like
   set_gpio_cfg( 12, NS_GPIO_CFG_FUNC_GPIO|NS_GPIO_CFG_OUTPUT ); */
#define set_gpio_cfg(pin,cfg) \
	{ \
		unsigned long flags; \
		spin_lock_irqsave( &ns_gpio_lock, flags );			\
		writel((readl (addr_gpio_cfg(pin)) & ~NS_GPIO_CFG_MASK(pin))| \
		       NS_GPIO_CFG_VAL((pin),(cfg)), addr_gpio_cfg(pin) );\
		spin_unlock_irqrestore( &ns_gpio_lock, flags );  \
	}

#define get_gpio_cfg(pin) \
({ \
	unsigned long flags;\
	unsigned long __v;\
	spin_lock_irqsave( &ns_gpio_lock, flags ); \
	__v = (readl(addr_gpio_cfg(pin)) & NS_GPIO_CFG_MASK(pin)) >> (((pin) % 8) * 4); \
	spin_unlock_irqrestore( &ns_gpio_lock, flags );\
	__v;\
})

#ifndef __ASSEMBLY__
# include <asm/arch/io.h>          /* rmwl() */
# include <linux/spinlock.h>	/* spin_lock_irqsave */
extern spinlock_t ns_gpio_lock;	/* use it to lock all GPIO accesses */
#endif /* __ASSEMBLY__ */

#define NS_GPIO_CFG_MASK(pin)	(NS_GPIO_CFG_VAL(pin, \
					 NS_GPIO_CFG_MA))
#define NS_GPIO_CFG_VAL(pin,cfg)	((cfg) << (((pin) % 8) * 4))

#define NS_GPIO_CFG_MA			(0x0F)
#define NS_GPIO_CFG_IO_MA		(0x08)
#define NS_GPIO_CFG_INPUT		(0x00)
#define NS_GPIO_CFG_OUTPUT		(0x08)
#define NS_GPIO_CFG_INV			(0x04)
#define NS_GPIO_CFG_FUNC_MA		(0x03)
#define NS_GPIO_CFG_FUNC_GPIO		(0x03)
#define NS_GPIO_CFG_FUNC_2		(0x02)
#define NS_GPIO_CFG_FUNC_1		(0x01)
#define NS_GPIO_CFG_FUNC_0		(0x00)

#define addr_gpio_ctrl(pin) ( NS_BBUS_GPIO_CTRL_BASE + ( ((pin) >> 5) *4) )
#define NS_GPIO_CTRL_MASK(pin)   (NS_GPIO_CTRL_VAL(pin,1))
#define NS_GPIO_CTRL_VAL(pin,cfg)        ((cfg) << ((pin) % 32) )

/*
 * To Read/Modify/Write a pin control register, use it like
 *    set_gpio_ctrl( 12, 1 );  GPIO12 Control bit ON, 0 for OFF
 */
#define set_gpio_ctrl(pin,ctrl) \
{ \
	spin_lock_irq( &ns_gpio_lock ); \
	writel( (readl(addr_gpio_ctrl(pin)) & ~NS_GPIO_CTRL_MASK(pin))|\
		NS_GPIO_CTRL_VAL((pin),(ctrl)), addr_gpio_ctrl(pin) ); \
	spin_unlock_irq( &ns_gpio_lock ); \
}

#define get_gpio_ctrl(pin) \
({ \
 	unsigned long __v;\
 	spin_lock_irq( &ns_gpio_lock ); \
 	__v = (readl(addr_gpio_ctrl(pin)) & NS_GPIO_CTRL_MASK(pin)) >> ((pin) % 32); \
 	spin_unlock_irq( &ns_gpio_lock ); \
 	__v;\
})

#define addr_gpio_stat(pin) ( NS_BBUS_GPIO_STAT_BASE + ( ((pin) >> 5) *4) )
#define NS_GPIO_STAT_MASK(pin) (1 << ((pin) %32) )
//#define get_gpio_stat(pin) (readl(addr_gpio_stat((pin))) & NS_GPIO_STAT_MASK((pin)) )
#define get_gpio_stat(pin) \
({ \
 	unsigned long __v;\
 	spin_lock_irq( &ns_gpio_lock );\
	__v = (readl(addr_gpio_stat(pin)) & NS_GPIO_STAT_MASK(pin)) >> ((pin) % 32);\
 	spin_unlock_irq( &ns_gpio_lock ); \
 	__v; \
 })

#define NS_BBUS_DMA_DESCR(ctrl,chan)	(NS_BBUS_MODULE_BASE+\
					 (ctrl)*0x110000+(chan)*0x20)
#define NS_BBUS_DMA_CTRL(ctrl,chan)	(NS_BBUS_MODULE_BASE+\
					 (ctrl)*0x110000+(chan)*0x20+0x10)
#define NS_BBUS_DMA_STAT(ctrl,chan)	(NS_BBUS_MODULE_BASE+\
					 (ctrl)*0x110000+(chan)*0x20+0x14)
/* the register addresses (utility) */

#define NS_BBUS_MASTER_RESET		(NS_BBUS_UTILITY_BASE + 0x00)
#define NS_BBUS_GPIO_CFG_BASE		(NS_BBUS_UTILITY_BASE + 0x10)
#define NS_BBUS_GPIO_CTRL_BASE		(NS_BBUS_UTILITY_BASE + 0x30)
#define NS_BBUS_GPIO_STAT_BASE		(NS_BBUS_UTILITY_BASE + 0x40)
#define NS_BBUS_MONITOR			(NS_BBUS_UTILITY_BASE + 0x50)
#define NS_BBUS_DMA_ISTAT		(NS_BBUS_UTILITY_BASE + 0x60)
#define NS_BBUS_DMA_IEN			(NS_BBUS_UTILITY_BASE + 0x64)
#define NS_BBUS_USB_CFG			(NS_BBUS_UTILITY_BASE + 0x70)
#define NS_BBUS_ENDIAN_CFG		(NS_BBUS_UTILITY_BASE + 0x80)
#define NS_BBUS_ARM_WAKE_UP		(NS_BBUS_UTILITY_BASE + 0x90)

/* register bit fields */

#define NS_BBUS_MASTER_RESET_UTIL	(0x00000100)
#define NS_BBUS_MASTER_RESET_I2C	(0x00000080)
#define NS_BBUS_MASTER_RESET_1284	(0x00000040)
#define NS_BBUS_MASTER_RESET_SER4	(0x00000020)
#define NS_BBUS_MASTER_RESET_SER3	(0x00000010)
#define NS_BBUS_MASTER_RESET_SER2	(0x00000008)
#define NS_BBUS_MASTER_RESET_SER1	(0x00000004)
#define NS_BBUS_MASTER_RESET_USB	(0x00000002)
#define NS_BBUS_MASTER_RESET_DMA	(0x00000001)

/* BS9750_BBUS_DMA_INT_BINT* are valid for *DMA_INT_STAT and *DMA_INT_ENABLE */

#define NS_BBUS_DMA_INT_BINT16		(0x00010000)
#define NS_BBUS_DMA_INT_BINT15		(0x00008000)
#define NS_BBUS_DMA_INT_BINT14		(0x00004000)
#define NS_BBUS_DMA_INT_BINT13		(0x00002000)
#define NS_BBUS_DMA_INT_BINT12		(0x00001000)
#define NS_BBUS_DMA_INT_BINT11		(0x00000800)
#define NS_BBUS_DMA_INT_BINT10		(0x00000400)
#define NS_BBUS_DMA_INT_BINT9 		(0x00000200)
#define NS_BBUS_DMA_INT_BINT8 		(0x00000100)
#define NS_BBUS_DMA_INT_BINT7 		(0x00000080)
#define NS_BBUS_DMA_INT_BINT6 		(0x00000040)
#define NS_BBUS_DMA_INT_BINT5 		(0x00000020)
#define NS_BBUS_DMA_INT_BINT4 		(0x00000010)
#define NS_BBUS_DMA_INT_BINT3 		(0x00000008)
#define NS_BBUS_DMA_INT_BINT2 		(0x00000004)
#define NS_BBUS_DMA_INT_BINT1 		(0x00000002)
#define NS_BBUS_DMA_INT_BINT0 		(0x00000001)

#define NS_BBUS_USB_CFG_OUTEN		(0x00000008)
#define NS_BBUS_USB_CFG_SPEED		(0x00000004)
#define NS_BBUS_USB_CFG_CFG_MA		(0x00000003)
#define NS_BBUS_USB_CFG_CFG_HOST_SOFT 	(0x00000003)
#define NS_BBUS_USB_CFG_CFG_DEVICE	(0x00000002)
#define NS_BBUS_USB_CFG_CFG_HOST	(0x00000001)
#define NS_BBUS_USB_CFG_CFG_DIS		(0x00000000)

#define NS_BBUS_ENDIAN_CFG_AHBM		(0x00001000)
#define NS_BBUS_ENDIAN_CFG_I2C		(0x00000080)
#define NS_BBUS_ENDIAN_CFG_IEEE1284	(0x00000040)
#define NS_BBUS_ENDIAN_CFG_SER4		(0x00000020)
#define NS_BBUS_ENDIAN_CFG_SER3		(0x00000010)
#define NS_BBUS_ENDIAN_CFG_SER2		(0x00000008)
#define NS_BBUS_ENDIAN_CFG_SER1		(0x00000004)
#define NS_BBUS_ENDIAN_CFG_USB		(0x00000002)
#define NS_BBUS_ENDIAN_CFG_DMA		(0x00000001)

#endif /* FS_NS9750_BBUS_H */
